package org.ids_mannheim;

import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.io.TempDir;

import static org.junit.jupiter.api.Assertions.*;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.PrintStream;
import java.nio.file.Path;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Logger;

public class PseudonymizeTest {
    private static final ByteArrayOutputStream outContent = new ByteArrayOutputStream();
    private static final ByteArrayOutputStream errContent = new ByteArrayOutputStream();
    private static final PrintStream originalOut = System.out;
    private static final PrintStream originalErr = System.err;

    @BeforeAll
    public static void setUpStreams() {
        System.setOut(new PrintStream(outContent));
        System.setErr(new PrintStream(errContent));
    }

    @AfterAll
    public static void restoreStreams() {
        System.setOut(originalOut);
        System.setErr(originalErr);
    }

    @BeforeAll
    public static void disableGrapeGrab() {
        System.setProperty("groovy.grape.enable", "false");
    }

    @Test
    void generateKeyTest() {
        GeneratePseudonymKey.main(new String[]{"src/test/resources/token_lemma_pos_freqs.tsv.xz"});
        assertTrue(outContent.toString().contains("\ndurchaus\t934\n"));
        assertTrue(outContent.toString().contains("\nlängst\t936\n"));
        assertFalse(outContent.toString().contains("\t937\n"));
    }

    @Test
    void readKeyTest() {
        ConcurrentHashMap map = (ConcurrentHashMap) Pseudonymize.readKey("src/test/resources/token.keys.gz", Logger.getLogger(""));
        assertEquals(936, map.get("längst"));
    }

    @TempDir
    static Path sharedTempDir;

    @Test
    void pseudonymizeFilesTest() {
        Logger log = Logger.getLogger("");
        String fname = sharedTempDir + File.separator + "token_lemma_pos_freqs.tsv.xz";
        new File(fname).delete();
        Pseudonymize.pseudonymizeFiles(new String[]{"src/test/resources/token_lemma_pos_freqs.tsv.xz"},
                new String[]{"src/test/resources/token.keys.gz", "src/test/resources/lemma.keys.gz"}, sharedTempDir.toAbsolutePath().toString(), log);
        ConcurrentHashMap pseudonymized = (ConcurrentHashMap) Pseudonymize.readKey(fname, log);
        assertEquals(223997, pseudonymized.get("936"));
    }
}